use std::fs::File;
use std::io::BufReader;
use std::path::{Path, PathBuf};
use std::time::Duration;
use rodio::{Decoder, OutputStream, Sink, Source};


//-----------------------------------------Structs




struct AudioPlayer {
    sink: Sink,
    playlist: Vec<SongMeta>,
}


//-----------------------------------Errors
#[derive(Debug, Clone)]
enum PlayerError {
    IndexNotFound,
    SomethingElse,
}

//-------------------------------------Impls




impl AudioPlayer {

    ///Returns a new empty player.
    fn new() -> Self {
        let (_stream, stream_handle) = OutputStream::try_default().unwrap();
        let sink = Sink::try_new(&stream_handle).unwrap();
        let playlist = vec![];

        Self {
            sink: sink,
            playlist: playlist,
        }
    }

    ///Creates a new player from an existing playlist.
    ///Does not consume the playlist.
    fn from_playlist(playlist: &Playlist) -> Self {
        let (_stream, stream_handle) = OutputStream::try_default().unwrap();
        let sink = Sink::try_new(&stream_handle).unwrap();

        Self {
            sink: sink,
            playlist: playlist.clone(),
        }
    }


    ///Appends a playlist to the current one, consuming it.
    fn add_playlist(&mut self, playlist: Vec<SongMeta>) {
        self.playlist.append(&mut playlist);
    }

    //renvoie un result, avance d'une chanson
    fn skip_forward(&mut self);
    //renvoie un result, recule d'une chanson
    fn skip_backwards(&mut self);
    //met en pause renvoie un result
    fn pause(&mut self);
    //Enleve la pause ou lit la première chanson de la playlist renvoie un result
    fn play(&mut self);
    //Change le volume envoie un result
    fn set_vol(&mut self, vol: f32);
    //Saute à la chanson donnée renvoie un result
    fn goto(&mut self, index: usize) -> Result<(), PlayerError>;
}

